data{
  int <lower=2> T;		//maximum length of series across all individuals
  int <lower=1> N;		//number of individuals
  int y_obs [N,T];	  //observed scores
  int y_len[N]; 		  //length of individual time series
  real y_mon[N];		  //time before treatment
  matrix [N,5] d;		  //dummy for treatment*agoraphobia (all 0 is no ag, cbt treatment)
  int ntot;           //total number of observations
  }

parameters{
  //slope
  vector [N] beta;		      //beta_n
  real beta0;			          //beta_0
  vector [5] beta1;		      //beta_1 vector
  real <lower=0> sig_beta;	//sd of beta_n
  //intercept
  vector [N] mu;			      //mu_n
  real mu0;					        //mu_0
  real mu1;					        //mu_1
  real <lower=0> sig_mu; 	  //sd of mu
  //moving average
  vector <lower=-1, upper=1> [N] psi;	//psi_n
  vector <upper=-0.00001> [N] X;		  //helpparameter prior psi
  //latent vector
  real <lower=0> sig_eps;		          //sd of innovation noise
  matrix [N,T] theta;			            //latent variable delta
  }

transformed parameters{
  matrix [N,T] eps;			      //noise of innovation
  for (n in 1:N){
    eps[n,1] <- theta[n,1];  //noise for t=1
    for (t in 2:y_len[n]){
      eps[n,t] <- theta[n,t]-theta[n,t-1]-beta[n]-psi[n]*eps[n,t-1];  //noise for t>1
    }
    for (t in (y_len[n]+1):T){
      eps[n,t] <- 1;
    }
  }
}
  

model{
  for (n in 1:N){
  //individual parameters
    beta[n] ~ normal(beta0+d[n]*beta1, sig_beta);
    mu[n] ~ normal(mu0+(mu1*y_mon[n]), sig_mu);
	//latent score for t=1
    theta[n,1] ~ normal(0, sig_eps);
	//latent score for t>1
	for (t in 2:y_len[n]){
		theta[n,t] ~ normal(theta[n,t-1]+beta[n]+eps[n,t-1]*psi[n], sig_eps);
	}
	//link function for t 
	for (t in 1:y_len[n]){
		//only if y_obs is non-missing (missing=99)
		if (y_obs[n,t] != 99){
			y_obs[n,t] ~ poisson(exp(theta[n,t]+mu[n]));
		}
    }
  }
  
#priors
beta0~ normal(0,5);
beta1~ normal(0,5);
mu0 ~ normal(0,5);
mu1 ~ normal(0,5);
sig_beta ~ normal(0.5,5);
sig_mu ~ normal(0.5,5);
sig_eps ~ normal(0.5,5);

	for (n in 1:N){
	//symmetrized reference prior for phi_n
		increment_log_prob(X[n] - log(1-(psi[n]*psi[n]))/2);
	}    
}


generated quantities{
//log-likelihoodcalculations
vector [ntot] loglik1;
vector [ntot] loglik2;
vector [ntot] loglik;
int i1;
int a;
int i2;
i1 <-1;

for (n in 1:N){
  for (t in 1:y_len[n]){
    loglik2[i1] <- poisson_log(y_obs[n,t], exp(theta[n,t]+mu[n]));
    i1 <- i1+1;
    }
}

i2<-1;
	for (n in 1:N){
		loglik1[i2] <- normal_log(theta[n,1], 0, sig_eps);
		i2 <-i2+1;
		for (t in 2:y_len[n]){
			loglik1[i2] <- normal_log(theta[n,t], theta[n,t-1]+beta[n]+psi[n]*eps[n,t-1], sig_eps);
			i2<-i2+1;
		}
	}
 
	for (i in 1:ntot){
     loglik[i] <- loglik1[i]+loglik2[i];
   }
}

